'use strict'

const fs = require('fs/promises')

const assetsFactory = require('./assetsFactory')

async function discover(assets, inputFolder) {
  let stats
  try {
    stats = await fs.stat(inputFolder)
  } catch (e) {
    // Do nothing
  }

  if (!stats || !stats.isDirectory()) {
    throw new Error(`${inputFolder} is not a valid folder`)
  }

  const files = await fs.readdir(inputFolder)

  for (const file of files) {
    const filePath = `${inputFolder}/${file}`
    const fileStats = await fs.stat(filePath)
    if (fileStats.isDirectory()) {
      assets.push(...(await discover([], filePath)))
    } else {
      assets.push(assetsFactory().fromFilepath(filePath))
    }
  }

  return assets
}

module.exports = ({ inputFolder }) => {
  const assets = []

  return {
    discover: () => discover(assets, inputFolder),
    assets: () => assets,
  }
}
