'use strict'
const fs = require('fs')

const sharp = require('sharp')

async function createImage(outputFile, combinations) {
  const images = combinations.map((trait) => ({
    input: trait.source,
  }))
  await sharp(images[0].input)
    .composite(images.slice(1, images.length))
    .withMetadata({
      density: 300,
    })
    .toFile(outputFile)
}

module.exports = ({ outputFolder }) => {
  if (!outputFolder) {
    throw new Error('Output folder is required')
  }
  outputFolder = `${outputFolder}/images`

  return {
    drawCollection: async (collection) => {
      if (fs.existsSync(outputFolder)) {
        fs.rmSync(outputFolder, { recursive: true, force: true })
      }

      fs.mkdirSync(outputFolder, { recursive: true })

      const items = collection.items()
      const imagePromises = []
      for (let i = 0; i < collection.count(); i++) {
        imagePromises.push(
          createImage(
            `${outputFolder}/${i + 1}.png`,
            collection.getAssets(items[i])
          )
        )
      }

      await Promise.all(imagePromises)
    },

    drawCombination: createImage,
  }
}
