'use strict'

function addGenerator(generators, name, resolver) {
  if (generators[name]) {
    throw new Error(`Can't register generator ${name} twice`)
  }
  generators[name] = resolver
}

module.exports = function generatorFactory() {
  const generators = {}

  addGenerator(
    generators,
    'traitRarityBased',
    require('./traitRarityGenerator')
  )
  return {
    register: (name, resolver) => {
      addGenerator(generators, name, resolver)
    },

    make: (type, options) => {
      if (!generators[type]) {
        throw new Error(`Invalid generator ${type}`)
      }

      const generator = generators[type](options)
      if (typeof generator.validateOptions === 'function') {
        generator.validateOptions()
      }
      return generator
    },
  }
}
