'use strict'

const Combination = require('../../entities/combination')

function getRandomRarity(rarities, randomizer) {
  const randomValue = randomizer.random()
  for (const rarity of rarities) {
    if (randomValue <= rarity.rate) {
      return rarity.id
    }
  }
}

function getRandomTrait(traits, randomizer) {
  const min = 0
  const max = traits.length - 1

  return traits[Math.floor(randomizer.random() * (max - min + 1) + min)]
}

module.exports = (rarities, traitMap, randomizer = {}) => {
  rarities = rarities.sort((r1, r2) => r2.rate - r1.rate)

  for (const index in rarities) {
    if (index > 0) {
      rarities[index].rate = parseFloat(
        (rarities[index].rate + rarities[index - 1].rate).toFixed(2)
      )
    }
  }

  return {
    generate: () => {
      randomizer.random = randomizer.random || Math.random
      const traits = Array.from(traitMap.values()).map((trait) => {
        const rarity = getRandomRarity(rarities, randomizer)
        const rarityTraits = trait.getAssetsByRarity(rarity)
        if (!rarityTraits || rarityTraits.length === 0) {
          throw new Error(
            `No traits with rarity ${rarity} and category ${trait.name}`
          )
        }
        return getRandomTrait(rarityTraits, randomizer)
      })

      return Combination(traits)
    },
  }
}
