'use strict'

const Drawer = require('../lib/drawers/index')
const MetadataWriter = require('../lib/metadataWriter')

const GeneratorFactory = require('./generators')
const DiscovererFactory = require('./discoverer')

module.exports = (options) => {
  if (!(options?.assets?.path || null)) {
    throw new Error('assets.path option is required')
  }
  if (!(options?.type || null)) {
    throw new Error('type option is required')
  }
  const generatorFactory = GeneratorFactory()

  const discoverer = DiscovererFactory({ inputFolder: options.assets.path })

  const drawer = Drawer(options)
  const metadataWriter = MetadataWriter(options)

  return {
    generate: async (supply) => {
      const assets = await discoverer.discover()
      const generator = generatorFactory.make(options.type, options)
      const collection = generator.generateCollection(assets, supply)
      return {
        collection,
        draw: () => {
          drawer.draw(collection)
        },
      }
    },
    drawer,
    metadataWriter,
  }
}
