'use strict'

const fs = require('fs')

function writeMetadata(outputFolder, assets, metadata, index) {
  const attributes = []
  for (const asset of assets) {
    attributes.push({
      trait_type: asset.trait,
      value: asset.label,
    })
  }
  const object = {
    name: `${metadata.name} #${index}`,
    attributes,
    image: `ipfs://REPLACE_ME/${index}.png`,
  }

  return fs.promises.writeFile(
    outputFolder,
    JSON.stringify(object, null, 4),
    'utf8'
  )
}

module.exports = (options) => {
  let { outputFolder } = options
  if (!outputFolder) {
    throw new Error('Output folder is required')
  }
  outputFolder = `${outputFolder}/metadata`

  return {
    writeCollection: async (collection) => {
      if (fs.existsSync(outputFolder)) {
        fs.rmSync(outputFolder, { recursive: true, force: true })
      }

      fs.mkdirSync(outputFolder, { recursive: true })

      const items = collection.items()
      const metadataPromises = []
      for (let i = 0; i < collection.count(); i++) {
        metadataPromises.push(
          writeMetadata(
            `${outputFolder}/${i + 1}.json`,
            collection.getAssets(items[i]),
            options.metadata,
            i + 1
          )
        )
      }

      await Promise.all(metadataPromises)
    },

    writeCombination: writeMetadata,
  }
}
